/* This file is part of ksirc
   Copyright (c) 2001 Malte Starostik <malte@kde.org>
             (c) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef MARKUP_H
#define MARKUP_H

#include <qstring.h>
#include <qmap.h>
#include <qvaluestack.h>
#include <qcolor.h>

namespace IRC
{

class Markup
{
public:
    // ### not sure all these belong into the same class. some are
    // for logical formatting (tag informational stuff) and others concrete 
    // formatting (make this text bold)

    static QString tagError( const QString &input );
    static QString tagInformation( const QString &input );
    static QString tagAction( const QString &input );

    static QString tagBold( const QString &input );
    static QString tagUnderlined( const QString &input );

    static QString tagImage( const QString &input, const QString &imageName );

    static QString tagColorCodes( const QString &input );

protected:
    Markup( const QString &input );

private:
    QString parseColorCodesToMarkup();

    QString pushTag( const QString &tag, const QString &attributes = QString::null );
    QString popTag( const QString &tag );
    QString toggleTag( const QString &tag );
    QString popAll();
    QColor ircColor( int color );

    QString m_input;

    QValueStack<QString> m_tags;
    QMap<QString, QString> m_attributes;
};

};

#endif // MARKUP_H
/* vim: et sw=4 ts=4
 */
