/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "mainwindow.h"
#include "message.h"

#include <kapplication.h>
#include <kcmdlineargs.h>

#include <stdlib.h>
#include <assert.h>

using namespace IRC;

static KCmdLineOptions options[] =
{
    { "+nick", "Nickname", 0 },
    { "+server", "Server", 0 },
    { "+[port]", "Port", 0 },
    { "+[password]", "Password", 0 },
    KCmdLineLastOption
};

int main( int argc, char **argv )
{
    IRC::initStatic();

    KCmdLineArgs::init( argc, argv, "messagetest", "irc test", "0.1" );

    KCmdLineArgs::addCmdLineOptions( options );

    KApplication::disableAutoDcopRegistration();

    KApplication app;

    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    IRC::ConnectionInfo info;

    if ( args->count() < 2 )
        KCmdLineArgs::usage();

    info.setUser( QString::fromLatin1( getenv( "USER" ) ) );
    info.setRealName( QString::fromLatin1( getenv( "USER" ) ) );
    info.setNick( QString::fromLatin1( args->arg( 0 ) ) );
    info.setHost( QString::fromLatin1( args->arg( 1 ) ) );

    if ( args->count() >= 3 )
        info.setPort( atoi( args->arg( 2 ) ) );
    else
        info.setPort( 6667 );

    if ( args->count() >= 4 )
        info.setPassword( args->arg( 3 ) );

    args->clear();

    MainWindow *mw = new MainWindow( info );
    mw->show();

    return app.exec();
}

/* vim: et sw=4 ts=4
 */

